import {ApiCall} from "tsrpc";
import { connGameLogDB } from "../../gameLog";
import {PeiJian, ReqGetList, ResGetList} from "../../shared/protocols/peijian/PtlGetList";
import { PeijianShared } from "../../shared/public/peijian";

export default async function (call: ApiCall<ReqGetList, ResGetList>) {
    let _maxLv = 0
    let color = {}
    let peijianCon = G.gc.peijian

    //所有上阵的英雄_id
    let heroIds = R.values(call.conn.gud.heroPos)
    //将所有未上阵的英雄的配件卸下
    if(heroIds.length>0){
        let _ids = heroIds.map(_id => G.mongodb.conversionId(_id));
        await G.mongodb.collection("hero").updateMany({uid: call.conn.uid},{
            $set:{
                peijian:{}
            }
        });
    }
    //如果配件穿戴在了未上阵的hero身上，则执行卸下操作
    await G.mongodb.collection('peijian').updateMany({uid: call.conn.uid, wearId: {$nin: heroIds}}, {$set: {wearId: ''}})
    
    
    let heroPeiJian = {};
    let dbList = (await G.mongodb.collection('peijian').find({uid: call.uid}).toArray()).map(p => {

        if(p.wearId){
            if(!heroPeiJian[ p.wearId ]){
                heroPeiJian[ p.wearId ] = {};
            }
            heroPeiJian[ p.wearId ][ Object.keys(heroPeiJian[ p.wearId ]).length + 1 ] = PeijianShared.fmt( p as any )
        }

        let np = G.mongodb.conversionIdObj(p);
        let {uid, ...ops} = np;

        if (np.lv > _maxLv) _maxLv = np.lv
        if (!color[peijianCon[np.peijianId].colour]) color[peijianCon[np.peijianId].colour] = 0
        color[peijianCon[np.peijianId].colour] = np.lv > color[peijianCon[np.peijianId].colour] ? np.lv : color[peijianCon[np.peijianId].colour]
        return ops;
    });

    //对hero的peijian字段进行修正
    if(Object.keys(heroPeiJian).length > 0){
        for(let heroid in heroPeiJian){
            await G.mongodb.collection("hero").updateOne({
                uid : call.conn.uid,
                _id : G.mongodb.conversionId(heroid)
            },{
                $set:{
                    peijian : heroPeiJian[heroid]
                }
            });
        }
    }

    // 记录玩家最大等级，颜色相关数据 注册任务用
    await G.mongodb.collection('playerInfo', 'usertasklog').updateOne({uid: call.conn.uid, type: 'usertasklog'},
        {$set: {maxpeijianlv: _maxLv, peijiancolor: color}}, {upsert: true})


    let list = Object.fromEntries(dbList.map(p => [G.formatRedisKey(p._id), p]));
    G.redis.set('peijian', call.uid, list);

    let recLshd = await G.mongodb.collection('playerInfo', 'lshd_peijian').findOne({
        uid: call.conn.uid,
        type: 'lshd_peijian'
    });
    let {uid, _id, type, ...peijians} = (recLshd || {});

    call.conn.lshd.peijian = peijians || {};
    call.succ({
        list: Object.fromEntries(dbList.map(p => [p._id, p])),
        lshd: peijians || {}
    });
}